#include <usercomng.h>
#include <subsys.h>
#include <confdb.h>
#include "ldapconf_defs.h"
#include "fields.h"

static HELP_FILE help_config ("ldapconf","formclient");

class USERINFO_COMNG: public USERACCT_COMNG{
	LDAPOBJECT ldap;
	CONFDB ldapdb;
	FIELD_DEFS userconf_ldapform;
	char *comng_enabled;
	const char *domain_profile;

	/*~PROTOBEG~ USERINFO_COMNG */
public:
	USERINFO_COMNG (DICTIONARY&_dict);
	int deluser (PRIVILEGE *);
	int save (PRIVILEGE *priv);
	void setupdia (DIALOG&dia);
	int validate (DIALOG&, int &nof);
	~USERINFO_COMNG (void);
	/*~PROTOEND~ USERINFO_COMNG */
};

PUBLIC USERINFO_COMNG::USERINFO_COMNG(
	DICTIONARY &_dict)
	: USERACCT_COMNG (_dict)
{	
    // Lookup domain profile
	const char *domain = dict.get_str("domain");
	char fpath[PATH_MAX]; 
	sprintf(fpath,"%s/%s",PROFILE_DIR,domain);

	if (!strcmp(domain,"/")) { // Maindomain profile
		domain_profile = "userinfo";
		D(debugf(4,"ldap_comng: main domain: %s\n",domain));
	}
	else if (fopen(fpath,"r")){ // Virtual domain profile
		domain_profile = domain;
		D(debugf(4,"ldap_comng: virtual domain: %s\n",domain));
	}
	else { // No profile
		comng_enabled = 0;
		return;
	}

	ldap.load_profile(domain_profile);
	if (!ldap.c_profile->getvalnum("profile","userconf_comng",0)) { // If profile comng is disabled
		comng_enabled = 0;
		return;
	}

	userconf_ldapform.c_form = ldap.form;
	userconf_ldapform.read();
	if (!dict.get_bool ("is_new")){
		ldap.filter.setfromf("uid=%s",dict.get_str("name"));
		ldap.search();
		ldap.export_confdb(&ldapdb);
		userconf_ldapform.loadval (&ldapdb,dict.get_str("name"));
	}
}

PUBLIC USERINFO_COMNG::~USERINFO_COMNG()
{
}

PUBLIC void USERINFO_COMNG::setupdia (
	DIALOG &dia)
{
	if (comng_enabled) {
		dia.newf_title (MSG_U(T_EXTRA,"LDAP"),1
						,"",MSG_R(T_EXTRA));
		userconf_ldapform.setupdia(dia);
	}
}	

PUBLIC int USERINFO_COMNG::save(
	PRIVILEGE *priv)
{
	if (!comng_enabled) return 0;
	const char *user;
	user = dict.get_str("name");
	char buf[256];
	char is_new;
	int n;
	/* Check if this entry exist */
	sprintf(buf,"uid=%s",user);
	ldap.filter.setfrom(buf);
	n = ldap.search();
	if ( n ) { is_new = 0; }
	else { is_new = 1; }

	/* Add DN */
	sprintf(buf,"uid=%s,%s,%s",user,ldap.dn_prefix.get(),ldap.bind.base.get());
	ldap.dn.setfrom(buf);

	/* Create a new entry */
	if (is_new){
		ldap.at_set("uid",user);
		ldap.at_set("cn",user); // For Netscape search
		ldap.oc_add("top");
		ldap.oc_add("account");
		ldap.oc_add("posixAccount");
		ldap.add();
	}

	/* Update the ldapobject to the ldap database */
	userconf_ldapform.saveval (&ldapdb,user);
	ldap.import_confdb(&ldapdb);
	return ldap.modify();
}

PUBLIC int USERINFO_COMNG::validate(
	DIALOG &,
	int &nof)
{
	int ret = 0;
	if (!comng_enabled) return 0;
	return ret;
}

PUBLIC int USERINFO_COMNG::deluser (
	PRIVILEGE *)
{
	int ret = 0;

	if (!comng_enabled) return 0;

	if (!dict.get_bool("is_new")){
		const char *user;
		user = dict.get_str("name");
		char buf[256];
		sprintf(buf,"uid=%s,%s,%s",user,ldap.dn_prefix.get(),ldap.bind.base.get());
		ldap.dn.setfrom(buf);
		ret = ldap.del();
	}
	return ret;
}

USERACCT_COMNG *ldapconf_newcomng(
	const char *key,
	DICTIONARY &dict)
{
	USERACCT_COMNG *ret = NULL;

	if (strcmp(key,"user")==0 && mode_ldap_userinfo){	
		ret = new USERINFO_COMNG (dict);
	}
	return ret;
}

static REGISTER_USERACCT_COMNG xxx (ldapconf_newcomng);

















